@@ -5,7 +5,7 @@ import tempfile
 import os
 import sys
 import ConfigParser
-from datetime import datetime
+from datetime import datetime, timedelta
 from getpass import getpass
 import logging
 import shelve
@@ -15,6 +15,7 @@ import socket
 import httplib
 import math
 import mimetypes
+import calendar
 
 from contextlib import closing # for Python2.6 compatibility
 
@@ -26,8 +27,9 @@ from boto.glacier.exceptions import UnexpectedHTTPResponseError
 from boto.exception import S3ResponseError
 from beefish import decrypt, encrypt_file
 import aaargh
+import grandfatherson
 
-__version__ = "0.3.7"
+__version__ = "0.3.8"
 
 DEFAULT_LOCATION = "us-east-1"
 DEFAULT_DESTINATION = "s3"
@@ -42,6 +44,7 @@ if not log.handlers:
 config = ConfigParser.SafeConfigParser({"default_destination": DEFAULT_DESTINATION})
 config.read(os.path.expanduser("~/.bakthat.conf"))
 
+
 class glacier_shelve(object):
     """
     Context manager for shelve
@@ -58,35 +61,45 @@ class glacier_shelve(object):
 
 class BakthatBackend:
     """Handle Configuration for Backends."""
-    def __init__(self, conf=None, extra_conf=[]):
+    def __init__(self, conf=None, extra_conf=[], section="aws"):
         self.custom_conf = None
         self.conf = {}
         if conf is None:
             try:
-                self.conf["access_key"] = config.get("aws", "access_key")
-                self.conf["secret_key"] = config.get("aws", "secret_key")
-                self.conf["region_name"] = config.get("aws", "region_name")
+                if section == "aws":
+                    self.conf["access_key"] = config.get(section, "access_key")
+                    self.conf["secret_key"] = config.get(section, "secret_key")
+                    self.conf["region_name"] = config.get(section, "region_name")
 
                 for key in extra_conf:
                     try:
-                        self.conf[key] = config.get("aws", key)
+                        self.conf[key] = config.get(section, key)
                     except Exception, exc:
                         log.exception(exc)
-                        log.error("Missing configuration variable")
+                        log.error("Missing configuration variable.")
                         self.conf[key] = None
 
             except ConfigParser.NoOptionError:
-                log.error("Configuration file not available.")
-                log.info("Use 'bakthat configure' to create one.")
+                if section == "aws":
+                    log.error("Configuration file not available.")
+                    log.info("Use 'bakthat configure' to create one.")
+                else:
+                    log.error("No {0} section available in configuration file.".format(section))
                 return
         else:
-            self.conf["access_key"] = conf.get("access_key")
-            self.conf["secret_key"] = conf.get("secret_key")
-            self.conf["region_name"] = conf.get("region_name", DEFAULT_LOCATION)
+            if section == "aws":
+                self.conf["access_key"] = conf.get("access_key")
+                self.conf["secret_key"] = conf.get("secret_key")
+                self.conf["region_name"] = conf.get("region_name", DEFAULT_LOCATION)
 
             for key in extra_conf:
                 self.conf[key] = conf.get(key)
 
+class RotationConfig(BakthatBackend):
+    def __init__(self, conf=None):
+        BakthatBackend.__init__(self, conf, 
+                                extra_conf=["days", "weeks", "months", "first_week_day"],
+                                section="rotation")
 
 class S3Backend(BakthatBackend):
     """
@@ -338,13 +351,17 @@ def _get_store_backend(conf, destination=DEFAULT_DESTINATION):
         destination = config.get("aws", "default_destination")
     return STORAGE_BACKEND[destination](conf)
 
+
 def _match_filename(filename, destination=DEFAULT_DESTINATION, conf=None):
+    if not filename:
+        raise Exception("Filename can't be blank")
     storage_backend = _get_store_backend(conf, destination)
 
     keys = [name for name in storage_backend.ls() if name.startswith(filename)]
     keys.sort(reverse=True)
     return keys
 
+
 def match_filename(filename, destination=DEFAULT_DESTINATION, conf=None):
     _keys = _match_filename(filename, destination, conf)
     regex_key = re.compile(r"(.+)(\d{14})\.tgz(\.enc)?")
@@ -361,6 +378,84 @@ def match_filename(filename, destination=DEFAULT_DESTINATION, conf=None):
     return keys
 
 
+def _interval_string_to_seconds(interval_string):
+    """Convert internal string like 1M, 1Y3M, 3W to seconds"""
+    interval_exc = "Bad interval format for {0}".format(interval_string)
+    interval_dict = {"s": 1, "m": 60, "h": 3600, "D": 86400,
+                       "W": 7*86400, "M": 30*86400, "Y": 365*86400}
+
+    interval_regex = re.compile("^(?P<num>[0-9]+)(?P<ext>[smhDWMY])")
+    seconds = 0
+
+    while interval_string:
+        match = interval_regex.match(interval_string)
+        if match:
+            num, ext = int(match.group("num")), match.group("ext")
+            if num > 0 and ext in interval_dict:
+                seconds += num * interval_dict[ext]
+                interval_string = interval_string[match.end():]
+            else:
+                raise Exception(interval_exc)
+        else:
+            raise Exception(interval_exc)
+    return seconds
+
+
+def _timedelta_total_seconds(td):
+    if hasattr(timedelta, "total_seconds"):
+        return getattr(td, "total_seconds")()
+
+    # Python 2.6 backward compatibility
+    return (td.microseconds + (td.seconds + td.days * 24 * 3600) * 10**6) / float(10**6)
+
+
+@app.cmd(help="Delete backups older than the given interval string.")
+@app.cmd_arg('-f', '--filename', type=str, default=os.getcwd())
+@app.cmd_arg('-i', '--interval', type=str, help="Interval string like 1M, 1W, 1M3W4h2s")
+@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier")
+def delete_older_than(filename, interval, destination=DEFAULT_DESTINATION, conf=None):
+    storage_backend = _get_store_backend(conf, destination)
+    interval_seconds = _interval_string_to_seconds(interval)
+
+    deleted = []
+
+    for key in match_filename(filename, destination, conf):
+        backup_age =  _timedelta_total_seconds(datetime.utcnow() - key.get("backup_date"))
+        if backup_age > interval_seconds:
+            real_key = key.get("key")
+            log.info("Deleting {0}".format(real_key))
+            storage_backend.delete(real_key)
+            deleted.append(real_key)
+
+    return deleted
+
+@app.cmd(help="Rotate backups using Grandfather-father-son backup rotation scheme.")
+@app.cmd_arg('-f', '--filename', type=str, default=os.getcwd())
+@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier")
+def rotate_backups(filename, destination=DEFAULT_DESTINATION, conf=None):
+    storage_backend = _get_store_backend(conf, destination)
+    rotate = RotationConfig(conf)
+
+    deleted = []
+
+    backups = match_filename(filename, destination, conf)
+    backups_date = [backup["backup_date"] for backup in backups]
+
+    to_delete = grandfatherson.to_delete(backups_date, days=int(rotate.conf["days"]),
+                                                    weeks=int(rotate.conf["weeks"]),
+                                                    months=int(rotate.conf["months"]),
+                                                    firstweekday=int(rotate.conf["first_week_day"]),
+                                                    now=datetime.utcnow())
+    
+    for key in backups:
+        if key.get("backup_date") in to_delete:
+            real_key = key.get("key")
+            log.info("Deleting {0}".format(real_key))
+            storage_backend.delete(real_key)
+            deleted.append(real_key)
+
+    return deleted
+
 @app.cmd(help="Backup a file or a directory, backup the current directory if no arg is provided.")
 @app.cmd_arg('-f', '--filename', type=str, default=os.getcwd())
 @app.cmd_arg('-d', '--destination', type=str, help="s3|glacier")
@@ -375,6 +470,8 @@ def backup(filename, destination=None, description=None, prompt="yes", **kwargs)
     date_component = datetime.utcnow().strftime("%Y%m%d%H%M%S")
     stored_filename = arcname +  date_component + ".tgz"
     
+    backup_data = dict(filename=arcname)
+
     password = kwargs.get("password")
     if password is None and prompt.lower() != "no":
         password = getpass("Password (blank to disable encryption): ")
@@ -390,6 +487,9 @@ def backup(filename, destination=None, description=None, prompt="yes", **kwargs)
 
         new_arcname = re.sub(r'(\.t(ar\.)?gz)', '', arcname)
         stored_filename = new_arcname + date_component + ".tgz"
+        
+        with open(outname) as outfile:
+            backup_data["size"] = os.fstat(outfile.fileno()).st_size
 
         bakthat_compression = False
     else:
@@ -398,11 +498,13 @@ def backup(filename, destination=None, description=None, prompt="yes", **kwargs)
             with closing(tarfile.open(fileobj=out, mode="w:gz")) as tar:
                 tar.add(filename, arcname=arcname)
             outname = out.name
-
+            out.seek(0)
+            backup_data["size"] = os.fstat(out.fileno()).st_size
         bakthat_compression = True
 
     bakthat_encryption = False
     if password:
+        bakthat_encryption = True
         log.info("Encrypting...")
         encrypted_out = tempfile.NamedTemporaryFile(delete=False)
         encrypt_file(outname, encrypted_out.name, password)
@@ -414,7 +516,12 @@ def backup(filename, destination=None, description=None, prompt="yes", **kwargs)
 
         outname = encrypted_out.name
 
-        bakthat_encryption = True
+        encrypted_out.seek(0)
+        backup_data["size"] = os.fstat(encrypted_out.fileno()).st_size
+
+
+    backup_data["metadata"] = dict(is_enc=bakthat_encryption)
+    backup_data["stored_filename"] = stored_filename
 
     log.info("Uploading...")
     storage_backend.upload(stored_filename, outname)
@@ -423,7 +530,8 @@ def backup(filename, destination=None, description=None, prompt="yes", **kwargs)
     if bakthat_encryption:
         os.remove(outname)
 
-    return True
+    log.debug(backup_data)
+    return backup_data
 
 
 @app.cmd(help="Give informations about stored filename, current directory if no arg is provided.")
@@ -444,6 +552,7 @@ def info(filename, destination=None, description=None, **kwargs):
                                                     str(len(keys))))
     return key
 
+
 @app.cmd(help="Set AWS S3/Glacier credentials.")
 def configure():
     config.add_section("aws")
@@ -460,6 +569,8 @@ def configure():
                 log.error("Invalid default_destination, should be s3 or glacier, try again.")
         else:
             default_destination = DEFAULT_DESTINATION
+            break
+
     config.set("aws", "default_destination", default_destination)
     config.set("aws", "glacier_vault", raw_input("Glacier Vault Name: "))
     region_name = raw_input("Region Name ({0}): ".format(DEFAULT_LOCATION))
@@ -470,6 +581,27 @@ def configure():
 
     log.info("Config written in %s" % os.path.expanduser("~/.bakthat.conf"))
 
+@app.cmd(help="Configure backups rotation")
+def configure_backups_rotation():
+    config.add_section("rotation")
+    config.set("rotation", "days", raw_input("Number of days to keep: "))
+    config.set("rotation", "weeks", raw_input("Number of weeks to keep: "))
+    config.set("rotation", "months", raw_input("Number of months to keep: "))
+    while 1:
+        first_week_day = raw_input("First week day (to calculate wich weekly backup keep, saturday by default): ")
+        if first_week_day:
+            if hasattr(calendar, first_week_day.upper()):
+                first_week_day = getattr(calendar, first_week_day.upper())
+                break
+            else:
+                log.error("Invalid first_week_day, please choose from sunday to saturday.")
+        else:
+            first_week_day = calendar.SATURDAY
+            break
+
+    config.set("rotation", "first_week_day", str(first_week_day))
+    config.write(open(os.path.expanduser("~/.bakthat.conf"), "w"))
+    log.info("Config written in %s" % os.path.expanduser("~/.bakthat.conf"))
 
 @app.cmd(help="Restore backup in the current directory.")
 @app.cmd_arg('-f', '--filename', type=str, default="")
@@ -560,9 +692,13 @@ def ls(destination=None, **kwargs):
     
     log.info(storage_backend.container)
 
-    for filename in storage_backend.ls():
+    ls_result = storage_backend.ls()
+
+    for filename in ls_result:
         log.info(filename)
 
+    return ls_result
+
 @app.cmd(help="Show Glacier inventory from S3")
 def show_glacier_inventory(**kwargs):
     if config.get("aws", "s3_bucket"):
@@ -573,6 +709,7 @@ def show_glacier_inventory(**kwargs):
     else:
         log.error("No S3 bucket defined.")
 
+
 @app.cmd(help="Show local Glacier inventory (from shelve file)")
 def show_local_glacier_inventory(**kwargs):
     conf = kwargs.get("conf", None)
@@ -580,6 +717,7 @@ def show_local_glacier_inventory(**kwargs):
     archives = glacier_backend.load_archives()
     log.info(json.dumps(archives, sort_keys=True, indent=4, separators=(',', ': ')))
 
+
 @app.cmd(help="Backup Glacier inventory to S3")
 def backup_glacier_inventory(**kwargs):
     conf = kwargs.get("conf", None)
@@ -597,5 +735,6 @@ def restore_glacier_inventory(**kwargs):
 def main():
     app.run()
 
+
 if __name__ == '__main__':
     main()
